__author__ = 'squirrel'

from model.algo.quaternions import *
from math import *


class MaxRadiusException(Exception):
    pass

class MinRadiusException(Exception):
    pass

def get_angles(coord, rotation, A, m, k, n, l, E):
    """
        coord = location of center of the robot
        rotation = rotation of base of the robot, quaternion here!
        A = point where is "alfa" axis compared to center of robot
        m = is 'x' offset of beta axis from alfa axis
        k = is 'y' offset of beta axis from alfa axis
        n = length of first 1/2 of leg
        l = length of second 1/2 of leg
        E = endpoint

        takes length of arms (n,l) and target point. Returns respective angles [alfa, beta, gama],  [0, 0,0] angles
        means that legs should be entirely on X+ axis from it's A point (it's joint is on A+[n, 0, 0] and E is
        A+[n+l, 0, 0])

        alfa is angle between X+ and first half of the leg on Z axis.
        beta is angle between X+ and first half of the leg on Y axis.
        gama is angle between line that goes from [0,0] to knee coord for another 1/2 of the leg
        """


    # first, we negate offset of the alfa axis from center of the robot
    base_E = (
        E[0] - A[0],
        E[1] - A[1],
        E[2] - A[2],
    )

    # now we negate the rotation of the robot with quaternions
    rotated_E = rotation.conjugate().v_multiply(base_E)


    # and lastly we negate the offset of the robot

    new_E = [
        rotated_E[0] - coord[0],
        rotated_E[1] - coord[1],
        rotated_E[2] - coord[2],
    ]

    # now we can just return result of simpler version of our function :)
    return centered_get_angles(m, k, n, l, new_E)


def centered_get_angles(m, k, n, l, E):
    """ simpler version of get_angles, asserts that origin is [0, 0, 0] and rotation of base is [0, 0, 0] """
    alfa = atan2(E[1], E[0])
    beta, gama = plane_get_angles(m, k, n, l, [sqrt(E[0]**2 + E[1]**2), E[2]])

    return alfa, beta, gama

def plane_get_angles(m, k, n, l, E):
    """ gets just beta/gama angles, and ignores existence of alfa axis, so it can be counted in "plane" """

    return centered_plane_get_angles(n, l, [E[0] - m, E[1] - k])

def centered_plane_get_angles(n, l, E):
    """  variation of "plane_get_angles", that asserts that base of the arm is in [0, 0] """

    t = sqrt(E[0]**2 + E[1]**2)

    if t> n + l:
        raise MaxRadiusException("Point is too far away")

    if t < n - l:
        raise MinRadiusException("point too close to base of the arm")

    beta = atan2(E[1], E[0])
    beta += acos(
        -(l**2 - n**2 - t**2)
        /
        (2*n*sqrt(t**2))
    )

    gama = acos(
        -(t**2 - n**2 - l**2)
        /
        (2*n*l)
    )

    gama -= pi
    return beta, gama



def test_centered_plane_get_angles():

    args_list = [
        ([5, 5, [10, 0]], [0, 0]),
        ([5, 5, [0, 10]], [90, 0]),
        ([5, 5, [5, 5]], [90,-90])
    ]

    for args, result in args_list:
        print("Testing for: {}".format(args))
        print("avaited result: {}".format(result))
        beta, gama = centered_plane_get_angles(*args)
        print(beta, gama)
        print(degrees(beta), degrees(gama))
        print()


def test_get_angles():
    # coord, rotation, A, m, k, n, l, E
    args_list = [
        ( {"coord": [0, 0, 10], "rotation": Quaternion.from_axisangle([0, 1, 0], 0),  "A": [5, 0, 0], "m":5, "k":5, "n": 5, "l": 5, "E": (20, 0, 15) }, (0, 0, 0) ),
        ( {"coord": [0, 0, 10], "rotation": Quaternion.from_axisangle([0, 1, 0], 0),  "A": [5, 0, 0], "m":5, "k":5, "n": 5, "l": 5, "E": (15, 0, 10) }, (0, 0, -90) )
    ]

    for args, result in args_list:
        print("Testing for: {}".format(args))
        print("avaited result: {}".format(result))
        alfa, beta, gama = get_angles(**args)
        print("radian angles:", alfa, beta, gama)
        print("degrees:", degrees(alfa),degrees(beta), degrees(gama))
        print()

#test_get_angles()