
from pycad import *
from pycad.tools.part import PartContainer
from pycad.tools.dimensions import AutomaticDimensionsMixin
from pycad.models.servo import Servo, HornArm
from pycad.algo.cad import neg, sum_vectors, rotate
from model.algo.quaternions import *
import model.algo.leg as algo

class Point(Sphere):

    @staticmethod
    def get_point_coords(source, target_id):

        if isinstance(source, list):
            coord = None

            for item in source:
                coord = Point.get_point_coords(item, target_id)
                if coord is not None:
                    break

            return coord

        if isinstance(source, Point):
            if source.id == target_id:
                return source.coord

        coord = Point.get_point_coords(source.children, target_id)

        if coord is not None:
            return sum_vectors(
                        rotate(
                            coord,
                            source.rotation
                        ),
                        source.coord
                    )
        else:
            return None

    def __init__(self, id, color=None):
        super().__init__(r=1)

        self._id = id

        if color is not None:
            self.color(color)

    @property
    def id(self):
        return self._id

class Tibia(PartContainer, AutomaticDimensionsMixin):

    length = 62.35 # measured by max_x :)

    def __init__(self):
        super().__init__()

        self.s = Servo.autofill("G90")

    def create_tibia(self, parts):
        parts.add(
            Point("tibia_base")
        )


        servo = self.s.copy()

        parts.add(

            Union().add(

                servo
                    .translate(neg(servo.pinion_offset))
                    .rotate([90, 0, 0]),
                Import("leg.stl")
                    .rotate([90, 0, 0])
                    .translate([22.35, servo.pinion_offset[2] - servo.mount_plate[2], 0])
                    .color("black"),

                Point("END", color="yellow")
                    .translate([self.length, 0, 0])

            )

            #Sphere(r=1).translate([self.length, 0, 0]).color("red")

        )

        return parts

    @property
    def mount_offset(self):
        return [0, self.min_y, 0]


class FemurArm(PartContainer):

    length = 55

    def create_arm(self, parts):

        h_single = HornArm(arm_n=1)
        h_double = HornArm(arm_n=2)

        parts.add(
            h_single.copy()
                .rotate([-90, 0, 180])
                .translate([self.length, 0, 0]),
            h_double
                .rotate([-90, 0, 0])
                .translate([self.length/2, 0 - h_double.base_height + h_double.arm_t*2, 0]),
            h_single
                .rotate([90, 0, 0])
                .translate([0, 0, 0])
        )

        return parts

class Femur(PartContainer):

    def __init__(self):
        super().__init__()

        self.t = Tibia()
        self.angle = 0
        self.arm = FemurArm()

    @property
    def length(self):
        return self.arm.length

    def set_angle(self, angle):
        self.angle = angle

    def create_femur(self, parts):
        parts.add(
            Point("femur_base")
        )

        t = self.t.copy()
        parts.add(
            t.rotate([0, -self.angle, 0])
                .translate([self.arm.length, 0, 0]),
            self.arm
        )

        return parts

class Coxa(PartContainer):

    def __init__(self):
        super().__init__()

        self.alfa = Servo.autofill("G90")
        self.beta = Servo.autofill("G90")


        self.f = Femur()
        self.angle = 0

        self._show_axis = False


    def show_axis(self):
        self._show_axis = True

    def set_angle(self, angle):
        self.angle = angle

    @property
    def m(self):
        return self.alfa.body[1]/2 + self.beta.body[1]/2

    @property
    def k(self):
        return -self.alfa.pinion_offset[2] - self.alfa.body_center[2] + self.beta.pinion_offset[0]

    def create_coxa(self, parts):
        parts.add(
            Point("coxa_base")
        )


        alfa = self.alfa.copy()
        beta = self.beta.copy()

        f = self.f.copy()

        parts.add(
            Union().add(

                alfa.translate(alfa.body_center),
                Union().add(
                    beta,
                    f.rotate([-90, 0, -90 + self.angle])
                        .translate(beta.pinion_offset)
                ).translate(alfa.body_center)
                    .rotate([0, -90, 0])
                    .translate([0, - self.m, 0 ])

            ).translate(neg(alfa.body_center))
                .translate(neg(self.alfa.pinion_offset))
                .rotate([0, 0, 90]),
        )

        if self._show_axis:
            print([self.m, 0, self.k])
            parts.add(
                Cylinder(d=1, h=100, center=True)
                    .color("blue"),
                Cylinder(d=1, h=100, center=True)
                    .color("green")
                    .rotate([90, 0, 0])
                    .translate([self.m, 0, self.k])
            )

        return parts

class Leg(PartContainer):

    def __init__(self, body, mirror):
        super().__init__()

        self.body = body
        self.target = [0,0,0]

        self.mirror = mirror

        self.coxa = Coxa()

    def shift(self, v):

        for i in range(3):
            self.target[i] += v[i]

    def move(self, v):

        self.target = v

    def create_parts(self, parts):


        alfa, beta, gama = algo.get_angles(
            coord=self.body.coord,
            rotation=Quaternion.from_axisangle([0, 1, 0], 0),
            A=self.coord,
            m = self.coxa.m,
            k = self.coxa.k,
            n = self.coxa.f.length,
            l = self.coxa.f.t.length,
            E = self.target
        )
        self.coxa.set_angle(degrees(beta))
        self.coxa.f.set_angle(degrees(gama))

        coxa = self.coxa.copy()

        if not self.mirror:
            parts.add(
                Union().add(
                    coxa
                ).rotate([0, 0, degrees(alfa)])
            )
        else:
            parts.add(
                Union().add(
                    coxa.mirror([0, 1, 0])
                ).rotate([0, 0, degrees(alfa)])
            )


        return parts
