#!/usr/bin/python3
__author__ = 'squirrel'


from pycad import *
from pycad.tools import *
from pycad.tools.part import PartContainer
from pycad.tools.dimensions import AutomaticDimensionsMixin
from pycad.models.servo import Servo, HornArm
from pycad.algo.cad import neg, sum_vectors, rotate
from model.algo.quaternions import *
import model.algo.leg as algo

from pycad.models.screws import FlatScrew

from leg import Leg, Point
from rpi import RaspiA

from  battery import Battery

from ubec import Ubec

class Body(Part):

    t = 2

    def __init__(self, tote):
        super().__init__()

        self.main_board = tote.main_board
        self.legs = tote.legs
        self.battery = tote.battery
        self.ubec = tote.ubec

        self.battery_offset = [0, 0, 0]
        self.main_board_offset = self.battery_offset[2] + self.battery.size[2] + self.t
        self.ubec_offset = [0, 0, -10]

        self.horn_arm = HornArm(arm_n=1)


    def create_part(self, part):

        negative = Union()

        for leg in self.legs:
            negative.add(
                self.horn_arm.get_hole()
                    .rotate([0, 0, 180 if leg.coord[0] > 0 else 0])
                    .translate(leg.coord)
            )

        for meta in self.main_board.screws_meta:
            negative.add(
                FlatScrew.autofill("M{}".format(meta['d']), min_l = 10)
                    .translate(meta['pos'])
                    .translate([0, 0, self.battery_offset[2] + self.battery.size[2]/2 + 2])
            )


        part.add(
            Difference().add(
                negative.debug(),
                Cube([100, 100, 0.1], center=True)
                    .translate([0, 0, self.battery_offset[2] + self.battery.size[2]/2])
            )
        )

        return part


class Tote(PartContainer):

    def __init__(self):
        super().__init__()

        self.translate([0, 0, 80])

        self.legs_positions = list()

        self.main_board = RaspiA()
        self.battery = Battery()
        self.ubec = Ubec()


        for x in [-25, 25]:
            for y in [-25, 25]:
                self.legs_positions.append([x, y, 0])

        self.legs = list()

        self.generate_minmax_legs()


        self.body = Body(self)

        self.main_board.translate([0, 0, self.body.main_board_offset])
        self.battery.translate(self.body.battery_offset)
        self.ubec.translate(self.body.ubec_offset)

    def generate_minmax_legs(self):

        for pos in self.legs_positions:
            for vector in [[2, 0, 0], [0, 2, 0]]:
                l = Leg(self, mirror = bool(pos[1] < 0) ^ bool(pos[0] < 0))
                self.legs.append(
                    l.translate(pos)
                )
                v = [0, 0, 0]
                for i in range(3):
                    v[i] = pos[i]*(vector[i] + 1)

                l.move(v)

    def generate_standart_legs(self):
        for pos in self.legs_positions:
            l = Leg(self, mirror = bool(pos[1] < 0) ^ bool(pos[0] < 0))
            self.legs.append(
                l.translate(pos)
            )

            pos_a = sqrt(pos[0]**2 + pos[1]**2 + pos[2]**2)
            v = [0, 0, 0]
            a = 60
            for i in range(3):
                v[i] = pos[i]*(pos_a + a)/pos_a

            l.move(v)

    def create_body(self, parts):

        parts.add(
           self.main_board,
            self.battery,
            self.body,
            self.ubec
        )

        for leg in self.legs:
            u = Union().add(
                leg,
                Point(id="BEND", color="blue")
                    .translate(leg.target)
                    .translate([
                    -self.coord[0],
                    -self.coord[1],
                    -self.coord[2]
                ])
                )

            parts.add(u)

            for point in ["coxa_base",  "femur_base", "tibia_base", "END", "BEND"]:

                print(point, Point.get_point_coords(u, point))

        return parts

Union().add(
    Tote()
).render(options={"fa": 2, "fs": 2})

p = [10, 10, 10]


